import { DxfDocument } from 'dxf-doc';
import { HatchPattern } from 'dxf-doc/entities/hatch';
import { Attributes, Point, Alignment, IRegion, IPath } from '..';
export declare abstract class AbstractDxfBuilder {
    protected document: DxfDocument;
    private ltypes;
    private patterns;
    private textStyles;
    constructor(document: DxfDocument);
    private getStyle;
    private getLType;
    private getLineWeight;
    private getColor;
    private getFillPattern;
    private getTextStyle;
    protected abstract convertY(y: number): number;
    line(x1: number, y1: number, x2: number, y2: number, attributes?: Attributes): void;
    rect(x: number, y: number, width: number, height: number, attributes?: Attributes | undefined): void;
    circle(cx: number, cy: number, r: number, attributes?: Attributes | undefined): void;
    ellipse(cx: number, cy: number, rx: number, ry: number, attributes?: Attributes | undefined): void;
    polyline(pts: Point[], attributes?: Attributes | undefined): void;
    polygon(pts: Point[], attributes?: Attributes | undefined): void;
    polybezier(pts: Point[], attributes?: Attributes | undefined): void;
    curve(pts: Point[], closed?: boolean | undefined, smoothing?: number | undefined, attributes?: Attributes | undefined): void;
    text(text: string, x: number, y: number, align: Alignment, attributes?: Attributes | undefined): void;
    region(region: IRegion, attributes?: Attributes | undefined): void;
    path(path: IPath, attributes?: Attributes | undefined): void;
    createRegion(): IRegion;
    createPath(): IPath;
    addPattern(pattern: HatchPattern): void;
}
