import { Attributes, Point, Alignment } from "..";
import { SvgNode } from "./svg-node";
import { SvgDrawing } from "./svg-drawing";
import { IRegion, IPath } from "../idrawing";
export declare abstract class AbstractSvgBuilder {
    root: SvgNode;
    private currentNode;
    constructor(root: SvgNode);
    line(x1: number, y1: number, x2: number, y2: number, attributes?: Attributes): void;
    rect(x: number, y: number, width: number, height: number, attributes?: Attributes): void;
    circle(cx: number, cy: number, r: number, attributes?: Attributes): void;
    ellipse(cx: number, cy: number, rx: number, ry: number, attributes?: Attributes): void;
    polyline(pts: Point[], attributes?: Attributes): void;
    polygon(pts: Point[], attributes?: Attributes): void;
    polybezier(pts: Point[], attributes?: Attributes): void;
    curve(pts: Point[], closed?: boolean, smoothing?: number, attributes?: Attributes): void;
    text(text: string, x: number, y: number, align: Alignment, attributes?: Attributes): void;
    region(region: IRegion, attributes?: Attributes): void;
    path(path: IPath, attributes?: Attributes): void;
    abstract createRegion(): IRegion;
    abstract createPath(): IPath;
    clip(region: IRegion, callback: () => void): void;
}
export declare class SvgBuilder extends AbstractSvgBuilder {
    drawing: SvgDrawing;
    constructor(drawing: SvgDrawing, root: SvgNode);
    createRegion(): IRegion;
    createPath(): IPath;
}
