"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var __1 = require("..");
var svg_node_1 = require("./svg-node");
var utils_1 = require("../utils");
var AbstractSvgBuilder = /** @class */ (function () {
    function AbstractSvgBuilder(root) {
        this.root = root;
        this.currentNode = root;
    }
    AbstractSvgBuilder.prototype.line = function (x1, y1, x2, y2, attributes) {
        attributes = attributes || {};
        attributes.x1 = x1;
        attributes.y1 = y1;
        attributes.x2 = x2;
        attributes.y2 = y2;
        this.currentNode.add(new svg_node_1.SvgNode('line', attributes));
    };
    AbstractSvgBuilder.prototype.rect = function (x, y, width, height, attributes) {
        attributes = attributes || {};
        attributes.x = x;
        attributes.y = y;
        attributes.width = width;
        attributes.height = height;
        this.currentNode.add(new svg_node_1.SvgNode('rect', attributes));
    };
    AbstractSvgBuilder.prototype.circle = function (cx, cy, r, attributes) {
        attributes = attributes || {};
        attributes.cx = cx;
        attributes.cy = cy;
        attributes.r = r;
        this.currentNode.add(new svg_node_1.SvgNode('circle', attributes));
    };
    AbstractSvgBuilder.prototype.ellipse = function (cx, cy, rx, ry, attributes) {
        attributes = attributes || {};
        attributes.cx = cx;
        attributes.cy = cy;
        attributes.rx = rx;
        attributes.ry = ry;
        this.currentNode.add(new svg_node_1.SvgNode('ellipse', attributes));
    };
    AbstractSvgBuilder.prototype.polyline = function (pts, attributes) {
        attributes = attributes || {};
        attributes.points = pts.reduce(function (acc, p) { return acc + " " + p.x + "," + p.y; }, '').trim();
        this.currentNode.add(new svg_node_1.SvgNode('polyline', attributes));
    };
    AbstractSvgBuilder.prototype.polygon = function (pts, attributes) {
        attributes = attributes || {};
        attributes.points = pts.reduce(function (acc, p) { return acc + " " + p.x + "," + p.y; }, '').trim();
        this.currentNode.add(new svg_node_1.SvgNode('polygon', attributes));
    };
    AbstractSvgBuilder.prototype.polybezier = function (pts, attributes) {
        attributes = attributes || {};
        attributes.d = pts.reduce(function (acc, point, i) { return i === 0
            ? "M" + point.x + "," + point.y
            : acc + " " + ((i - 1) % 3 === 0 ? 'C' : '') + point.x + "," + point.y; }, '');
        this.currentNode.add(new svg_node_1.SvgNode('path', attributes));
    };
    AbstractSvgBuilder.prototype.curve = function (pts, closed, smoothing, attributes) {
        if (closed === void 0) { closed = false; }
        if (smoothing === void 0) { smoothing = 0.25; }
        pts = utils_1.Geometry.curveToPolybezier(pts, closed, smoothing);
        this.polybezier(pts, attributes);
    };
    AbstractSvgBuilder.prototype.text = function (text, x, y, align, attributes) {
        attributes = attributes || {};
        attributes.x = x;
        attributes.y = y;
        attributes['text-anchor'] = __1.Alignment[align].toLowerCase();
        var textNode = new svg_node_1.SvgNode('text', attributes);
        textNode.add(text);
        this.currentNode.add(textNode);
    };
    AbstractSvgBuilder.prototype.region = function (region, attributes) {
        var _this = this;
        this.clip(region, function () {
            var bounds = region.getBounds();
            attributes = attributes || {};
            attributes.style = "stroke: none";
            _this.rect(bounds.x1, bounds.y1, bounds.x2 - bounds.x1, bounds.y2 - bounds.y1, attributes);
        });
    };
    AbstractSvgBuilder.prototype.path = function (path, attributes) {
        var svgPath = path;
        attributes = attributes || {};
        var pathNode = new svg_node_1.SvgNode('use', __assign(__assign({}, attributes), { 'xlink:href': "#" + svgPath.id }));
        this.currentNode.add(pathNode);
    };
    AbstractSvgBuilder.prototype.clip = function (region, callback) {
        var svgRegion = region;
        var g = new svg_node_1.SvgNode('g', { mask: "url(#" + svgRegion.id + ")" });
        this.currentNode.add(g);
        var oldCurrentNode = this.currentNode;
        this.currentNode = g;
        try {
            callback();
        }
        finally {
            this.currentNode = oldCurrentNode;
        }
    };
    return AbstractSvgBuilder;
}());
exports.AbstractSvgBuilder = AbstractSvgBuilder;
var SvgBuilder = /** @class */ (function (_super) {
    __extends(SvgBuilder, _super);
    function SvgBuilder(drawing, root) {
        var _this = _super.call(this, root) || this;
        _this.drawing = drawing;
        return _this;
    }
    SvgBuilder.prototype.createRegion = function () {
        return this.drawing.createRegion();
    };
    SvgBuilder.prototype.createPath = function () {
        return this.drawing.createPath();
    };
    return SvgBuilder;
}(AbstractSvgBuilder));
exports.SvgBuilder = SvgBuilder;
