"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var PolygonClipping = __importStar(require("polygon-clipping"));
var __1 = require("..");
var utils_1 = require("../utils");
var svg_node_1 = require("./svg-node");
var SvgPath = /** @class */ (function () {
    function SvgPath(id) {
        this.id = id;
        this.polygons = [];
        this.currentPolygon = [];
        this.root = new svg_node_1.SvgNode('path', { id: id, d: '' });
    }
    SvgPath.prototype.move = function (x, y) {
        this.root.attributes.d += "M " + x + "," + y;
        this.currentPoint = { x: x, y: y };
        this.initialPoint = { x: x, y: y };
        this.currentPolygon = [[x, y]];
        this.polygons.push([this.currentPolygon]);
    };
    SvgPath.prototype.line = function (x, y) {
        if (!this.currentPoint) {
            throw new Error('Invalid operation. No current point to start path command');
        }
        this.root.attributes.d += "L " + x + "," + y;
        this.currentPoint = { x: x, y: y };
        this.currentPolygon.push([x, y]);
    };
    SvgPath.prototype.polyline = function (pts) {
        var _this = this;
        if (!this.currentPoint) {
            throw new Error('Invalid operation. No current point to start path command');
        }
        pts.forEach(function (p) {
            _this.root.attributes.d += "L " + p.x + "," + p.y;
            _this.currentPolygon.push([p.x, p.y]);
        });
        this.currentPoint = { x: pts[pts.length - 1].x, y: pts[pts.length - 1].y };
    };
    SvgPath.prototype.polybezier = function (pts) {
        var _a;
        var _this = this;
        if (!this.currentPoint) {
            throw new Error('Invalid operation. No current point to start path command');
        }
        if (pts.length === 0 || pts.length % 3 !== 0) {
            throw new Error("Invalid points length: " + pts.length);
        }
        pts.forEach(function (p, i) { return _this.root.attributes.d += i % 3 === 0 ? "C " + p.x + "," + p.y : " " + p.x + "," + p.y; });
        pts.unshift(this.currentPoint);
        this.currentPoint = { x: pts[pts.length - 1].x, y: pts[pts.length - 1].y };
        pts = utils_1.Geometry.approximatePolybezier(pts, __1.DRAWING_SETTINGS.APPROX_CURVE_MAX_DEVIATION);
        (_a = this.currentPolygon).push.apply(_a, pts.map(function (p) { return [p.x, p.y]; })); // Don't use concat, the currentPolygon reference should not change
    };
    SvgPath.prototype.curve = function (pts, smoothing) {
        if (!this.currentPoint) {
            throw new Error('Invalid operation. No current point to start path command');
        }
        pts = pts.slice();
        pts.unshift(this.currentPoint);
        pts = utils_1.Geometry.curveToPolybezier(pts, false, smoothing).slice(1);
        this.polybezier(pts);
    };
    SvgPath.prototype.close = function () {
        if (!this.initialPoint) {
            throw new Error('Can\'t close path. No initial point established');
        }
        this.root.attributes.d += "Z";
        this.currentPoint = this.initialPoint;
        this.currentPolygon.push([this.initialPoint.x, this.initialPoint.y]);
        this.currentPolygon = [[this.initialPoint.x, this.initialPoint.y]];
        this.polygons.push([this.currentPolygon]);
    };
    SvgPath.prototype.getPolygons = function () {
        return this.polygons.length <= 1 ? this.polygons : PolygonClipping.union.apply(PolygonClipping, __spreadArrays([this.polygons[0]], this.polygons.slice(1)));
    };
    return SvgPath;
}());
exports.SvgPath = SvgPath;
