"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var PolygonClipping = __importStar(require("polygon-clipping"));
var __1 = require("..");
var svg_builder_1 = require("./svg-builder");
var svg_node_1 = require("./svg-node");
var SvgRegionData = /** @class */ (function () {
    function SvgRegionData(drawing, root, fill) {
        this.fill = fill;
        this.polygons = [];
        this.builder = new svg_builder_1.SvgBuilder(drawing, root);
    }
    SvgRegionData.prototype.addPoint = function (x, y) {
        if (!this.bounds) {
            this.bounds = { x1: x, y1: y, x2: x, y2: y };
        }
        else {
            this.bounds.x1 = Math.min(x, this.bounds.x1);
            this.bounds.y1 = Math.min(y, this.bounds.y1);
            this.bounds.x2 = Math.max(x, this.bounds.x2);
            this.bounds.y2 = Math.max(y, this.bounds.y2);
        }
    };
    SvgRegionData.prototype.ptsToPolygon = function (pts) {
        return [pts.map(function (p) { return [p.x, p.y]; })];
    };
    SvgRegionData.prototype.rect = function (x, y, width, height) {
        this.addPoint(x, y);
        this.addPoint(x + width, y + height);
        this.builder.rect(x, y, width, height, { stroke: 'none', fill: this.fill });
        this.polygons.push([[[x, y], [x + width, y], [x + width, y + height], [x, y + height]]]);
    };
    SvgRegionData.prototype.circle = function (cx, cy, r) {
        this.addPoint(cx - r, cy - r);
        this.addPoint(cx + r, cy + r);
        this.builder.circle(cx, cy, r, { stroke: 'none', fill: this.fill });
        this.polygons.push(this.ptsToPolygon(__1.Geometry.approximateEllipse(cx, cy, r, r, __1.DRAWING_SETTINGS.APPROX_ELLIPSE_LINES)));
    };
    SvgRegionData.prototype.ellipse = function (cx, cy, rx, ry) {
        this.addPoint(cx - rx, cy - ry);
        this.addPoint(cx + rx, cy + ry);
        this.builder.ellipse(cx, cy, rx, ry, { stroke: 'none', fill: this.fill });
        this.polygons.push(this.ptsToPolygon(__1.Geometry.approximateEllipse(cx, cy, rx, ry, __1.DRAWING_SETTINGS.APPROX_ELLIPSE_LINES)));
    };
    SvgRegionData.prototype.polygon = function (pts) {
        var _this = this;
        pts.forEach(function (x) { return _this.addPoint(x.x, x.y); });
        this.builder.polygon(pts, { stroke: 'none', fill: this.fill });
        this.polygons.push(this.ptsToPolygon(pts));
    };
    SvgRegionData.prototype.closedCurve = function (pts, smoothing) {
        var _this = this;
        pts = __1.Geometry.curveToPolybezier(pts, true, smoothing);
        var approxPts = __1.Geometry.approximatePolybezier(pts, __1.DRAWING_SETTINGS.APPROX_CURVE_MAX_DEVIATION);
        approxPts.forEach(function (x) { return _this.addPoint(x.x, x.y); });
        this.builder.polybezier(pts, { stroke: 'none', fill: this.fill });
        this.polygons.push(this.ptsToPolygon(approxPts));
    };
    SvgRegionData.prototype.path = function (path) {
        var _a;
        var _this = this;
        this.builder.path(path, { stroke: 'none', fill: this.fill });
        var svgPath = path;
        var pathPolygons = svgPath.getPolygons();
        pathPolygons.forEach(function (polygon) {
            polygon.forEach(function (ring) {
                ring.forEach(function (pair) { return _this.addPoint(pair[0], pair[1]); });
            });
        });
        (_a = this.polygons).push.apply(_a, pathPolygons);
    };
    SvgRegionData.prototype.getBounds = function () {
        return this.bounds || { x1: 0, y1: 0, x2: 0, y2: 0 };
    };
    SvgRegionData.prototype.getPolygons = function () {
        return this.polygons.length <= 1 ? this.polygons : PolygonClipping.union.apply(PolygonClipping, __spreadArrays([this.polygons[0]], this.polygons.slice(1)));
    };
    return SvgRegionData;
}());
var SvgRegion = /** @class */ (function () {
    function SvgRegion(drawing, id) {
        this.drawing = drawing;
        this.id = id;
        this.root = new svg_node_1.SvgNode('mask', { id: id });
        this.includes = new SvgRegionData(drawing, this.root, '#fff');
        this.excludes = new SvgRegionData(drawing, this.root, '#000');
    }
    SvgRegion.prototype.include = function (callback) {
        callback(this.includes);
    };
    SvgRegion.prototype.exclude = function (callback) {
        callback(this.excludes);
    };
    SvgRegion.prototype.getBounds = function () {
        return this.includes.getBounds();
    };
    SvgRegion.prototype.outline = function () {
        var _this = this;
        var polygons = PolygonClipping.difference(this.includes.getPolygons(), this.excludes.getPolygons());
        return polygons.map(function (polygon) {
            return polygon.map(function (ring) {
                var path = _this.drawing.createPath();
                if (ring.length) {
                    path.move(ring[0][0], ring[0][1]);
                    path.polyline(ring.slice(1).map(function (pair) { return ({ x: pair[0], y: pair[1] }); }));
                    path.close();
                }
                return path;
            });
        }).reduce(function (acc, paths) { return acc.concat(paths); }, []);
    };
    return SvgRegion;
}());
exports.SvgRegion = SvgRegion;
