"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Geometry = /** @class */ (function () {
    function Geometry() {
    }
    Geometry.curveControlPoint = function (current, previous, next, reverse, smoothing) {
        var p = previous || current;
        var n = next || current;
        var angle = Math.atan2(n.y - p.y, n.x - p.x) + (reverse ? Math.PI : 0);
        var length = Math.sqrt(Math.pow(n.x - p.x, 2) + Math.pow(n.y - p.y, 2)) * smoothing;
        var x = current.x + Math.cos(angle) * length;
        var y = current.y + Math.sin(angle) * length;
        return { x: x, y: y };
    };
    Geometry.curveControlPoints = function (point, i, a, smoothing, closed) {
        var p2 = closed && i < 2 ? a[i + a.length - 2] : a[i - 2];
        var p1 = closed && i < 1 ? a[i + a.length - 1] : a[i - 1];
        var n = closed && i >= a.length - 1 ? a[i - a.length + 1] : a[i + 1];
        var cps = Geometry.curveControlPoint(p1, p2, point, false, smoothing);
        var cpe = Geometry.curveControlPoint(point, p1, n, true, smoothing);
        return [cps, cpe, point];
    };
    Geometry.curveToPolybezier = function (pts, closed, smoothing) {
        if (smoothing === void 0) { smoothing = 0.25; }
        var result = [];
        result = pts.reduce(function (acc, point, i, a) { return i === 0 ? [point] : acc.concat(Geometry.curveControlPoints(point, i, a, smoothing, closed)); }, result);
        if (closed && pts.length) {
            var arr = pts.slice(1);
            arr.push(pts[0]);
            result = result.concat(Geometry.curveControlPoints(arr[arr.length - 1], arr.length - 1, arr, smoothing, closed));
        }
        return result;
    };
    Geometry.approximateEllipse = function (cx, cy, rx, ry, count) {
        var result = new Array(count);
        for (var i = 0; i < count; i++) {
            var a = Math.PI * 2 / count * i;
            result[i] = {
                x: Math.cos(a) * rx + cx,
                y: Math.sin(a) * ry + cy
            };
        }
        return result;
    };
    Geometry.approximatePolybezier = function (pts, maxDeviation) {
        var beziers = pts.reduce(function (acc, p, i, a) {
            return i % 3 === 0 && i !== a.length - 1 ? acc.concat([[a[i], a[i + 1], a[i + 2], a[i + 3]]]) : acc;
        }, []);
        return beziers.reduce(function (acc, bezier, i) {
            return acc.concat(Geometry.approximateBezier(bezier, maxDeviation, i === beziers.length - 1));
        }, []);
    };
    Geometry.approximateBezier = function (bezier, maxDeviation, includeLast) {
        var result = [bezier[0]];
        var recursiveBezier = function (p1, p2, p3, p4) {
            var p12 = { x: (p1.x + p2.x) / 2, y: (p1.y + p2.y) / 2 };
            var p23 = { x: (p2.x + p3.x) / 2, y: (p2.y + p3.y) / 2 };
            var p34 = { x: (p3.x + p4.x) / 2, y: (p3.y + p4.y) / 2 };
            var p123 = { x: (p12.x + p23.x) / 2, y: (p12.y + p23.y) / 2 };
            var p234 = { x: (p23.x + p34.x) / 2, y: (p23.y + p34.y) / 2 };
            var p1234 = { x: (p123.x + p234.x) / 2, y: (p123.y + p234.y) / 2 };
            var dx = p4.x - p1.x;
            var dy = p4.y - p1.y;
            var d2 = Math.abs(((p2.x - p4.x) * dy - (p2.y - p4.y) * dx));
            var d3 = Math.abs(((p3.x - p4.x) * dy - (p3.y - p4.y) * dx));
            if ((d2 + d3) * (d2 + d3) < (maxDeviation * maxDeviation) * ((dx * dx) + (dy * dy))) {
                result.push(p1234);
                return;
            }
            recursiveBezier(p1, p12, p123, p1234);
            recursiveBezier(p1234, p234, p34, p4);
        };
        if (bezier[0].x !== bezier[1].x || bezier[0].x !== bezier[2].x || bezier[0].x !== bezier[3].x) {
            recursiveBezier(bezier[0], bezier[1], bezier[2], bezier[3]);
        }
        if (includeLast) {
            result.push(bezier[3]);
        }
        return result;
    };
    return Geometry;
}());
exports.Geometry = Geometry;
